@@ -60,7 +60,7 @@ parseCommandLine (int argc, const char ** argv,
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
     optEntry * option_def;
-        /* Instructions to optParseOptions3 on how to parse our options.
+        /* Instructions to pm_optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
 
@@ -84,7 +84,7 @@ parseCommandLine (int argc, const char ** argv,
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    optParseOptions3(&argc, (char **)argv, opt, sizeof(opt), 0);
+    pm_optParseOptions3(&argc, (char **)argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
     free(option_def);
@@ -248,7 +248,7 @@ main(int argc, const char ** argv) {
 
         const char * matrixOpt;
 
-        asprintfN(&matrixOpt, "-matrix=%s", matrixOptValue);
+        pm_asprintf(&matrixOpt, "-matrix=%s", matrixOptValue);
 
         pm_message("Running Pnmconvol -normalize %s", matrixOpt);
 
@@ -256,9 +256,9 @@ main(int argc, const char ** argv) {
                      "pnmconvol", matrixOpt, cmdline.inputFilespec,
                      "-normalize", "-quiet", plainOpt, NULL);
 
-        strfree(matrixOpt);
+        pm_strfree(matrixOpt);
     }
-    strfree(matrixOptValue);
+    pm_strfree(matrixOptValue);
 
     return 0;
 }
